Client error types for the Haystack client library.
ClientError covers all failure modes that can occur during client operations.
Enums
- Client
Error - Errors that can occur during Haystack client operations.
Client error types for the Haystack client library.
ClientError covers all failure modes that can occur during client operations.